Package: svyROC
Title: Estimation of the ROC Curve and the AUC for Complex Survey Data
Version: 1.1.0
Authors@R: 
    c(person("Amaia", "Iparragirre", , "amaia.iparragirre@ehu.eus", role = c("aut", "cre", "cph"),
             comment = c(ORCID = "0000-0002-0660-6535")),
      person("Irantzu", "Barrio", role = c("aut")),
      person("Inmaculada", "Arostegui", role = c("aut")))
Maintainer: Amaia Iparragirre <amaia.iparragirre@ehu.eus>
Description: Estimate the receiver operating characteristic (ROC) curve, area under the curve (AUC) and optimal cut-off points for individual classification taking into account complex sampling designs when working with complex survey data. Methods implemented in this package are described in: A. Iparragirre, I. Barrio, I. Arostegui (2024) <doi:10.1002/sta4.635>; A. Iparragirre, I. Barrio, J. Aramendi, I. Arostegui (2022) <doi:10.2436/20.8080.02.121>; A. Iparragirre, I. Barrio (2024) <doi:10.1007/978-3-031-65723-8_7>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Imports: survey, svyVarSel
NeedsCompilation: no
Packaged: 2026-04-13 09:40:24 UTC; amaiaiparragirre
Author: Amaia Iparragirre [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-0660-6535>),
  Irantzu Barrio [aut],
  Inmaculada Arostegui [aut]
Repository: CRAN
Date/Publication: 2026-04-14 10:20:07 UTC
Built: R 4.6.0; ; 2026-04-26 15:26:46 UTC; unix
