# Generated by roxygen2: do not edit by hand

export(ci.wauc)
export(corrected.wauc)
export(ht.indep)
export(ht.paired)
export(wauc)
export(wocp)
export(wroc)
export(wroc.plot)
export(wse)
export(wsp)
import(survey)
import(svyVarSel)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,quasibinomial)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,weights)
