#' Simulated data
#'
#' This dataset has been simulated in order to provide the users with an example dataset.
#'
#' @format ## `example_variables_wroc_2`
#' A data frame with 1720 rows and 10 columns:
#' \describe{
#'   \item{y}{Response variable}
#'   \item{x1,...,x6}{Covariates}
#'   \item{strata}{Strata variable}
#'   \item{cluster}{Cluster variable}
#'   \item{weights}{Sampling weights}
#'   ...
#' }
"example_variables_wroc_2"
