#' Simulated data
#'
#' This dataset has been simulated in order to provide the users with an example dataset.
#'
#' @format ## `example_data_wroc`
#' A data frame with 740 rows and 3 columns:
#' \describe{
#'   \item{y}{Response variable}
#'   \item{phat}{Predicted probabilities}
#'   \item{weights}{Sampling weights}
#'   ...
#' }
"example_data_wroc"
