% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_variables_wroc.R
\docType{data}
\name{example_variables_wroc}
\alias{example_variables_wroc}
\title{Simulated data}
\format{
\subsection{\code{example_variables_wroc}}{

A data frame with 1720 rows and 10 columns:
\describe{
\item{y}{Response variable}
\item{x1,...,x6}{Covariates}
\item{strata}{Strata variable}
\item{cluster}{Cluster variable}
\item{weights}{Sampling weights}
...
}
}
}
\usage{
example_variables_wroc
}
\description{
This dataset has been simulated in order to provide the users with an example dataset.
}
\keyword{datasets}
